#!/bin/bash
#

mgbimg=mgb.rbf
s2mimg=S2Me.rbf
quadimg=quadE1.rbf
brboximg=brbox.rbf

me=$(readlink -f $0)
BASE=$(dirname ${me})

if ! which omgstartilid > /dev/null; then
    echo "Error: omgstartilid misssing. Perform omgupdate first."
    exit 2
fi
if ! which omgstopall > /dev/null; then
    omgstartilid 2>&1 >/dev/null
    echo "Error: omgstopall misssing. Perform omgupdate first."
    exit 2
fi
if ! which wdstop > /dev/null; then
    echo "Error: wdstop missing. Perform omgupdate first."
    exit 2
fi
if ! which cfgFlash > /dev/null; then
    echo "Error: cfgFlash  misssing. Perform omgupdate first."
    exit 2
fi

omgstartilid 2>&1 >/dev/null

echo "Check hardware for FPGA update ..."

isS2Me=$(if lspci -n | grep -q "1011:f001"; then echo 1; else echo 0;fi)
isMGB=$(if lspci -n | grep -q "1011:f002"; then echo 1; else echo 0;fi)
isQuadE1=$(if lspci -n | grep -q "1011:f004"; then echo 1; else echo 0;fi)
isAtomBox=$(if lspci -n | grep -q "1011:f005"; then echo 1; else echo 0;fi)

if [ "${isS2Me}" != "1"  -a "${isMGB}" != "1" -a "${isQuadE1}" != "1" -a "${isAtomBox=}" != "1" ]; then
    echo "Hardware without FPGA."
    exit 0;
fi

OSType=$(bash -c "if [ -r /etc/redhat-release ] ; then cat /etc/redhat-release | head -n 1 | cut -d ' ' -f 1 | sed 's/.*/\L&/' ; elif [ -r /etc/issue ] ; then cat /etc/issue | head -n 1 | cut -d ' ' -f 1 | sed 's/.*/\L&/'; else echo unknown ; fi")

if [ $OSType = "centos" ]; then
	echo "CentOS detected"
elif [ $OSType = "red" ]; then
	echo "RedHat detected"
elif [ $OSType = "ubuntu" ]; then
	echo "Ubuntu detected"
        echo "Check software..."
        if ! dpkg -s omg-userfs >/dev/null 2>&1; then
            echo "Error: userfs missing. Perform omgupdate first."
            exit 2
        fi
        if ! dpkg -s omg-modules >/dev/null 2>&1; then
            echo "Error: modules missing. Perform omgupdate first."
            exit 2
        fi
        if ! dpkg -s omg-utilities >/dev/null 2>&1; then
            echo "Error: utilities missing. Perform omgupdate first."
            exit 2
        fi
else
	echo "unknown OS"
        exit 3
fi

for i in $(getchannels -a 2>&1| grep '^0\.[a-z]/X/CFG');do
    hw=$(echo $i | cut -d '.' -f 2 | cut -d '/' -f 1)
    case "${hw}" in
    "a")
        device="/dev/tdmSpi8" 
        ;;
    "b")
        device="/dev/tdmSpi108" 
        ;;
    "c")
        device="/dev/tdmSpi208" 
        ;;
    "d")
        device="/dev/tdmSpi308" 
        ;;
    "e")
        device="/dev/tdmSpi408" 
        ;;
    "f")
        device="/dev/tdmSpi508" 
        ;;
    "g")
        device="/dev/tdmSpi608" 
        ;;
    "h")
        device="/dev/tdmSpi708" 
        ;;
    *)
        echo "cant calculate apropiate device!"
        exit 3
        ;;
    esac
    
    let version=0x$(echo fpga_version | confTool /dev/ilid ${i}| head -1| cut -f 3 -d ' ')
    if [ -z "${version}" ]; then
	echo "Error: Cannot detect fpga version."
	exit 1
    fi

    if [ "${isS2Me}" == "1" ]; then
        echo "it's an S2Me"
        let versiontoflash=0x0109
        let minversiontoflash=0x0106
    elif [ "${isMGB}" = "1" ]; then
        echo "it's an MGB"
        let versiontoflash=0x010c
        let minversiontoflash=0x10c
    elif [ "${isQuadE1}" = "1" ]; then
        echo "it's an 4S2M"
        let versiontoflash=0x010d
        let minversiontoflash=0x010a
    elif [ "${isAtomBox}" == "1" ]; then
        echo "it's an OMGA"
        let versiontoflash=0x010d
        let minversiontoflash=0x010c
    else
        echo "Error: Can't flash due to wrong hardware."
        exit 1;
    fi

    printf "            version is: %04X\n" ${version}
    printf "   version to flash is: %04X\n" ${versiontoflash}
    printf "good enough version is: %04X\n" ${minversiontoflash}

    if [ ${version} -eq 0 ];then
        echo "Error: Can't flash due to can't get version."
        exit 1;
    fi

    if [ "${1}" == "update" -a ${version} -lt ${versiontoflash} ]; then
        flashit=1
    fi

    if [ "${1}" == "forceupdate" ]; then
        flashit=1
    fi

    if [ -z "${version}" -o ${version} -lt ${minversiontoflash} ];then
        flashit=1
    fi

    if [ "${flashit}" == "1" ];then
        if [ ! -e /tmp/flashfpga ]; then
            read -p "You need an FPGA update. After update it is absolutely necessary to powercycle your box. [Continue/Abort/Default=Abort]!" answer
            if [ "${answer}" != "Continue" -a "${answer}" != "C" -a "${answer}" != "c" ]; then
                echo "Aborting"
                exit 1
            fi
            echo
        fi

        echo "Start flashing $i FPGA. DON'T POWER OFF OR RESTART!"
        echo "Stopping software..."
        wdstop 2>/dev/null
        omgstopall
        omgstartilid 2>&1 >/dev/null

        if [ "${isAtomBox}" == "1" ]; then
            echo "Save old OMGA image"
            cfgFlash ${device} r fpgaold.bin
            ${BASE}/fpgadiff ${BASE}/${brboximg} fpgaold.bin
            if [ $? -eq 0 ]; then
                echo "image is already flashed --> do nothing."
            else
		echo "Erase old and flash new image"
		cfgFlash ${device} w ${BASE}/${brboximg} 
		echo "Reread new OMGA image"
		cfgFlash ${device} r fpga.bin
		echo "Check new OMGA image"
		${BASE}/fpgadiff ${BASE}/${brboximg} fpga.bin
		if [ $? -ne 0 ]; then
                    echo "Error: Check failed images differ; please try again without reboot!!!"
                    echo "       otherwise there is a good chance for sending broken hardware to Ferrari!!!"
		else
                    echo "Flashing OMGA FPGA done"
                    echo "Now a powercycle is needed."
                    touch /tmp/powercyle
		fi
	    fi
        elif [ "${isS2Me}" == "1" ]; then
            echo "Save old S2Me image"
            cfgFlash ${device} r fpgaold.bin
            ${BASE}/fpgadiff ${BASE}/${s2mimg} fpgaold.bin
            if [ $? -eq 0 ]; then
                echo "image is already flashed --> do nothing."
            else
		echo "Erase old and flash new S2Me image"
		cfgFlash ${device} w ${BASE}/${s2mimg} 
		echo "Reread new S2Me image"
		cfgFlash ${device} r fpga.bin
		echo "Check new S2Me image"
		${BASE}/fpgadiff ${BASE}/${s2mimg} fpga.bin
		if [ $? -ne 0 ]; then
                    echo "Error: Check failed images differ; please try again without reboot!!!"
                    echo "       otherwise there is a good chance for sending broken hardware to Ferrari!!!"
		fi
		echo "Flashing S2Me FPGA done"
		echo "Now a powercycle is needed."
		touch /tmp/powercyle
	    fi
        elif [ "${isMGB}" = "1" ]; then 
            echo "Save old MGB image"
            cfgFlash ${device} r fpgaold.bin
            ${BASE}/fpgadiff ${BASE}/${mgbimg} fpgaold.bin
            if [ $? -eq 0 ]; then
                echo "image is already flashed --> do nothing."
            else
		echo "Erase old and flash new image"
		cfgFlash ${device} w ${BASE}/${mgbimg} 
		echo "Reread new MGB image"
		cfgFlash ${device} r fpga.bin
		echo "Check new MGB image"
		${BASE}/fpgadiff ${BASE}/${mgbimg} fpga.bin
		if [ $? -ne 0 ]; then
                    echo "Error: Check failed images differ; please try again without reboot!!!"
                    echo "       otherwise there is a good chance for sending broken hardware to Ferrari!!!"
		fi
		echo "Flashing MGB FPGA done"
		echo "Now a powercycle is needed."
		touch /tmp/powercyle
	    fi
        elif [ "${isQuadE1}" = "1" ]; then 
            echo "Save old 4S2M image"
            cfgFlash ${device} r fpgaold.bin
            ${BASE}/fpgadiff ${BASE}/${quadimg} fpgaold.bin
            if [ $? -eq 0 ]; then
                echo "image is already flashed --> do nothing."
            else
		echo "Erase old and flash new 4S2M image"
		cfgFlash ${device} w ${BASE}/${quadimg} 
		echo "Reread new 4S2M image"
		cfgFlash ${device} r fpga.bin
		echo "Check new 4S2M image"
		${BASE}/fpgadiff ${BASE}/${quadimg} fpga.bin
		if [ $? -ne 0 ]; then
                    echo "Error: Check failed images differ; please try again without reboot!!!"
                    echo "       otherwise there is a good chance for sending broken hardware to Ferrari!!!"
		fi
		echo "Flashing 4S2m FPGA done"
		echo "Now a powercycle is needed."
		touch /tmp/powercyle
	    fi
        else
            echo "Error: Can't flash due to wrong hardware."
        fi
    fi
done

sleep 5




 
